<script></script>
<li class="dropdown globaltoc-container">
    <a
        href="{{ pathto(master_doc) }}"
        class="dropdown-toggle"
        data-toggle="dropdown"
        aria-label="{{master_doc}}"
        >This Chapter<b class="caret"></b
    ></a>
    <ul class="dropdown-menu globaltoc">
        {% raw %} 
        {% for row in subchapter_list %}
        <li class="toctree-l1">
            <a class="reference internal" href="{{ row['subchap_uri']}}"
                >{{ row['title']}}</a
            >
        </li>
        {% endfor %} 
        {% endraw %}
    </ul>
</li>
