<script>
</script>
<li class="dropdown globaltoc-container">
  <a href="{{ pathto(master_doc) }}"
     class="dropdown-toggle"
     data-toggle="dropdown" aria-label="{{master_doc}}">{{ theme_navbar_site_name }} <b class="caret"></b></a>
  <ul class="dropdown-menu globaltoc">
      {{ toctree(maxdepth=theme_globaltoc_depth|toint, collapse=False, includehidden=theme_globaltoc_includehidden|tobool) }}
  </ul>
</li>
