.exercises{background-color:azure}figcaption{font:italic 13px/18px Cambria,Georgia,Times New Roman,Times,serif;margin:.75em 0;text-align:center}img.bookfig{margin-left:auto;margin-right:auto}.cl_caption{font-weight:700;text-align:center}.runestone iframe{display:block;margin-left:auto;margin-right:auto}
/*# sourceMappingURL=prefix-915.e93fea9b3e714de5.css.map*/