.ptx-runestone-container,.runestone-sphinx {
    @import (less) url("./activecode.css");
}

:root {
    --ac-out-background: lightgray;
    --ac-failed: rgb(222, 142, 150);
    --ac-passed: rgb(131, 211, 130);
}

:root.dark-mode {
    --ac-out-background: var(--code-inline, #2e2e2e);
    --ac-failed: #592d2d;
    --ac-passed: rgb(40, 88, 40);
}