/* Major sections */
.webgldemo_container {
  background-color: #fcf8e3;
  box-sizing: border-box;
  width: 100%;
  display: block;
  margin-bottom: 1em;
}

.webgldemo_canvas {
  background-color: #fcf8e3;
  box-sizing: border-box;
}

.webgldemo_controls {
}
