.tab-pane{padding:20px 15px 10px}.nav.nav-tabs{background-color:var(--componentBgColor);border-color:var(--componentBorderColor);padding-top:6px}.nav.nav-tabs>li{margin-left:6px;margin-right:1px}.nav.nav-tabs>li>a{border:1px solid var(--componentBorderColor);border-radius:4px 4px 0 0;color:var(--bodyFont,#555)}.nav.nav-tabs>li.active>a,.nav.nav-tabs>li.active>a:is(:hover,:focus),.nav.nav-tabs>li>a:is(:hover,:focus){background-color:#156ba4;color:#fff}
/*# sourceMappingURL=prefix-442.6ae0ea584f8e75c5.css.map*/