"use strict";(self.webpackChunkWebComponents=self.webpackChunkWebComponents||[]).push([[640],{45640:(e,n,o)=>{o.r(n),o(29389);var s=o(30998),t=o.n(s);window.WaveSkin=t(),$(document).on("runestone:login-complete",window.WaveDrom.ProcessAll)}}]);
//# sourceMappingURL=640.js.map