.ptx-runestone-container,.runestone-sphinx {
    @import (less) url("./dragndrop.css");
}

:root {
    --draggableBgColor: #EFEFEF;
    --dropableBgColor: #DDDDDD;
    --draggableIncorrectBorder: red;
    --draggableIncorrectBg: #f2dede;
}

:root.dark-mode,
[data-theme="dark"] {
    --draggableBgColor: #393434;
    --dropableBgColor:  #2a2d30;
    --draggableIncorrectBorder: red;
    --draggableIncorrectBg: #6d2d2d;
}