.clickable{cursor:pointer}.clickable:has(code){display:inline-block;padding:3px}.clickable-clicked:has(code){box-shadow:0 0 2px 2px rgba(0,100,0,.75)}.clickable-clicked{background-color:var(--clickableBgColor)}.clickable-clicked td{background-color:#dad6c1;box-shadow:none}table{border-collapse:collapse}td.clickable,th.clickable{border:1px solid #000}td.clickable-clicked{background-color:#dad6c1;box-shadow:none}td img{margin:5px}.clickable-clicked th,th.clickable-clicked{background-color:#dad6c1;box-shadow:none}.clickable-incorrect,.clickable-incorrect td,.clickable-incorrect th{background-color:var(--clickableIncorrectBgColor);border:1px solid red!important}img.clickable-incorrect{box-shadow:0 0 2px 2px rgba(100,0,0,.75)}.clickable-incorrect:has(code){box-shadow:0 0 2px 2px rgba(251,0,0,.75)}:root{--clickableBgColor:#ff0;--clickableIncorrectBgColor:#fcc}:root.dark-mode{--clickableBgColor:#3c5624;--clickableIncorrectBgColor:#562c2c;.clickable-clicked:has(code):not(.clickable-incorrect){box-shadow:0 0 2px 2px rgba(251,255,21,.75)}}
/*# sourceMappingURL=prefix-198.86b6b2a70a576922.css.map*/