
.CodeMirror {
    font-family: Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace;
}

// Force some colors to match those used by PreTeXt for syntax highlighting
.cm-s-default .cm-comment { color: #3a812e; }
.cm-s-default .cm-keyword { color: #0679B7; }
.cm-s-default .cm-variable { color: #5e1ca0; }

// Values pulled from codemirror/theme/blackboard.css
:root.dark-mode {
    .ptx-runestone-container,.runestone-sphinx {
        .cm-s-default.CodeMirror { background: #141414; color: #F8F8F8; }
        .cm-s-default div.CodeMirror-selected { background: #364f95; }
        .cm-s-default .CodeMirror-line::selection, .cm-s-default .CodeMirror-line > span::selection, .cm-s-default .CodeMirror-line > span > span::selection { background: rgba(37, 59, 118, .99); }
        .cm-s-default .CodeMirror-line::-moz-selection, .cm-s-default .CodeMirror-line > span::-moz-selection, .cm-s-default .CodeMirror-line > span > span::-moz-selection { background: rgba(37, 59, 118, .99); }
        .cm-s-default .CodeMirror-gutters { background: #0C1021; border-right: 0; }
        .cm-s-default .CodeMirror-guttermarker { color: #FBDE2D; }
        .cm-s-default .CodeMirror-guttermarker-subtle { color: #888; }
        .cm-s-default .CodeMirror-linenumber { color: #888; }
        .cm-s-default .CodeMirror-cursor { border-left: 1px solid #A7A7A7; }

        .cm-s-default .cm-keyword { color: #2d94fb; }
        .cm-s-default .cm-atom { color: #abc792; }
        .cm-s-default .cm-number { color: #abc792; }
        .cm-s-default .cm-def { color: #8DA6CE; }
        .cm-s-default .cm-variable { color: #e3e1c2; }
        .cm-s-default .cm-operator { color: #F8F8F8; }
        .cm-s-default .cm-comment { color: #68a950; }
        .cm-s-default .cm-string { color: #ca9147; }
        .cm-s-default .cm-string-2 { color: #ca9147; }
        .cm-s-default .cm-meta { color: #D8FA3C; }
        .cm-s-default .cm-builtin { color: #2d94fb; }
        .cm-s-default .cm-tag { color: #8DA6CE; }
        .cm-s-default .cm-attribute { color: #8DA6CE; }
        .cm-s-default .cm-header { color: #cad18a; }
        .cm-s-default .cm-hr { color: #AEAEAE; }
        .cm-s-default .cm-link { color: #8DA6CE; }
        .cm-s-default .cm-error { background: #9D1E15; color: #F8F8F8; }

        .cm-s-default .CodeMirror-activeline-background { background: #3C3636; }
        .cm-s-default .CodeMirror-matchingbracket { outline:1px solid grey;color:white !important; }

        [lang="cpp"] {
            .cm-s-default .cm-type { color: rgb(45, 148, 251); }
        }
    }
    .CodeMirror__locked-line {
        background-color: #222;
    }
    .CodeMirror__gutter-locked-marker {
        background-color: #888;
    }
}