.hparsons_section{clear:both;margin-left:auto;margin-right:auto;max-width:800px;position:relative}.hparsons_section>:not(.hparsons_section){margin-left:auto;margin-right:auto;max-width:500pt}.hp_question{margin:5px;padding-left:10px;padding-top:10px}.hp_actions{text-align:center}.hp_output{background-color:inherit;display:none;max-width:450px}.hp_output pre,.hp_sql_result{background-color:var(--questionBgColor,#d3d3d3)}.hp_sql_result{border-radius:6px;margin-bottom:10px;padding:10px}.hp_sql_result_success{color:var(--parsonsCorrectBgColor,green)}.hp_sql_result_failure,.hp_sql_result_success{background-color:transparent;border:0;margin-bottom:10px;margin-top:10px;min-height:0!important;padding:0}.hp_sql_result_failure{color:var(--parsonsIncorrectBgColor,red)}micro-parsons .drag-area{background-color:var(--questionBgColor,#efefff);border:1px solid var(--componentBorderColor,#efefff)}micro-parsons .parsons-block{color:#000}pre code.hljs{display:block;overflow-x:auto;padding:1em}code.hljs{padding:3px 5px}.hljs{background:#fff;color:#000}.xml .hljs-meta{color:silver}.hljs-comment,.hljs-quote{color:#007400}.hljs-attribute,.hljs-keyword,.hljs-literal,.hljs-name,.hljs-selector-tag,.hljs-tag{color:#aa0d91}.hljs-template-variable,.hljs-variable{color:#3f6e74}.hljs-code,.hljs-meta .hljs-string,.hljs-string{color:#c41a16}.hljs-link,.hljs-regexp{color:#0e0eff}.hljs-bullet,.hljs-number,.hljs-symbol,.hljs-title{color:#1c00cf}.hljs-meta,.hljs-section{color:#643820}.hljs-built_in,.hljs-class .hljs-title,.hljs-params,.hljs-title.class_,.hljs-type{color:#5c2699}.hljs-attr{color:#836c28}.hljs-subst{color:#000}.hljs-formula{background-color:#eee;font-style:italic}.hljs-addition{background-color:#baeeba}.hljs-deletion{background-color:#ffc8bd}.hljs-selector-class,.hljs-selector-id{color:#9b703f}.hljs-doctag,.hljs-strong{font-weight:700}.hljs-emphasis{font-style:italic}.hparsons-input{padding:15px}.hparsons-tip{font-style:italic}.parsons-tooltip{background-color:#000;border-radius:6px;color:#fff;max-width:120px;padding:5px;position:absolute;text-align:center;top:30px;visibility:hidden;width:max-content;z-index:1}.parsons-block:hover .parsons-tooltip{visibility:visible}.parsons-block{background-color:#efefef;border:1px solid #d3d3d3;border-radius:10px;display:inline-block;font-family:monospace;margin:5px 1px 0;padding:5px 10px;position:relative}.drop-area .parsons-block.incorrectPosition{background-color:#ffbaba;border:1px solid red}.parsons-block:focus,.parsons-block:hover{border-color:#000}.drop-area{background-color:#ffa;height:auto;margin:2px 0;min-height:32px;padding:0 5px 5px}.drop-area.incorrect{background-color:#f2dede;border-color:#f2b6b6}.drop-area.correct{background-color:#dff0d8;border-color:#ade595}.drag-area .parsons-block:hover .tooltip{visibility:visible}.drag-area{background-color:#efefff;height:auto;margin:2px 0;min-height:32px;padding:0 5px 5px}
/*# sourceMappingURL=prefix-486.99971b51d1eb59cf.css.map*/