#component-preview{margin-right:20px;max-height:100%;overflow-y:scroll;padding-top:60px;position:fixed;right:0;top:0}.toggle-preview{background-color:azure;border:4px solid;border-radius:10px;box-shadow:10px 5px 5px gray;padding-top:10px}
/*# sourceMappingURL=prefix-380.67748940131a4a84.css.map*/