(()=>{"use strict";var e,t,r,a,n,o={},i={};function c(e){var t=i[e];if(void 0!==t)return t.exports;var r=i[e]={id:e,loaded:!1,exports:{}};return o[e].call(r.exports,r,r.exports,c),r.loaded=!0,r.exports}c.m=o,e=[],c.O=(t,r,a,n)=>{if(!r){var o=1/0;for(s=0;s<e.length;s++){for(var[r,a,n]=e[s],i=!0,d=0;d<r.length;d++)(!1&n||o>=n)&&Object.keys(c.O).every((e=>c.O[e](r[d])))?r.splice(d--,1):(i=!1,n<o&&(o=n));if(i){e.splice(s--,1);var l=a();void 0!==l&&(t=l)}}return t}n=n||0;for(var s=e.length;s>0&&e[s-1][2]>n;s--)e[s]=e[s-1];e[s]=[r,a,n]},c.n=e=>{var t=e&&e.__esModule?()=>e.default:()=>e;return c.d(t,{a:t}),t},c.d=(e,t)=>{for(var r in t)c.o(t,r)&&!c.o(e,r)&&Object.defineProperty(e,r,{enumerable:!0,get:t[r]})},c.f={},c.e=e=>Promise.all(Object.keys(c.f).reduce(((t,r)=>(c.f[r](e,t),t)),[])),c.u=e=>e+".js",c.miniCssF=e=>"prefix-"+e+"."+{131:"b184379d0dc5871b",168:"07f19b71041a3105",186:"0d47de180767fda9",198:"86b6b2a70a576922",249:"6ca19da539a8ed1a",294:"38995dcf76812d5a",327:"d7a9b0f03a195d7f",350:"47ffc57177ab6f53",380:"67748940131a4a84",412:"5f52f40663e4ea71",442:"6ae0ea584f8e75c5",460:"7c0b70502d117f7d",486:"99971b51d1eb59cf",576:"9b1852878840651d",580:"22958d4327b0425a",616:"5c78f604ddad1ec0",621:"1beebe7935b0310d",773:"514769d5f67b595c",819:"98b1f6bebc93239c",824:"271ae6cff6401ed5",877:"aee8829759a9a26a",915:"e93fea9b3e714de5"}[e]+".css",c.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),c.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),t={},r="WebComponents:",c.l=(e,a,n,o)=>{if(t[e])t[e].push(a);else{var i,d;if(void 0!==n)for(var l=document.getElementsByTagName("script"),s=0;s<l.length;s++){var f=l[s];if(f.getAttribute("src")==e||f.getAttribute("data-webpack")==r+n){i=f;break}}i||(d=!0,(i=document.createElement("script")).charset="utf-8",i.timeout=120,c.nc&&i.setAttribute("nonce",c.nc),i.setAttribute("data-webpack",r+n),i.src=e),t[e]=[a];var u=(r,a)=>{i.onerror=i.onload=null,clearTimeout(b);var n=t[e];if(delete t[e],i.parentNode&&i.parentNode.removeChild(i),n&&n.forEach((e=>e(a))),r)return r(a)},b=setTimeout(u.bind(null,void 0,{type:"timeout",target:i}),12e4);i.onerror=u.bind(null,i.onerror),i.onload=u.bind(null,i.onload),d&&document.head.appendChild(i)}},c.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},c.nmd=e=>(e.paths=[],e.children||(e.children=[]),e),(()=>{var e;c.g.importScripts&&(e=c.g.location+"");var t=c.g.document;if(!e&&t&&(t.currentScript&&"SCRIPT"===t.currentScript.tagName.toUpperCase()&&(e=t.currentScript.src),!e)){var r=t.getElementsByTagName("script");if(r.length)for(var a=r.length-1;a>-1&&(!e||!/^http(s?):/.test(e));)e=r[a--].src}if(!e)throw new Error("Automatic publicPath is not supported in this browser");e=e.replace(/#.*$/,"").replace(/\?.*$/,"").replace(/\/[^\/]+$/,"/"),c.p=e})(),a=e=>new Promise(((t,r)=>{var a=c.miniCssF(e),n=c.p+a;if(((e,t)=>{for(var r=document.getElementsByTagName("link"),a=0;a<r.length;a++){var n=(i=r[a]).getAttribute("data-href")||i.getAttribute("href");if("stylesheet"===i.rel&&(n===e||n===t))return i}var o=document.getElementsByTagName("style");for(a=0;a<o.length;a++){var i;if((n=(i=o[a]).getAttribute("data-href"))===e||n===t)return i}})(a,n))return t();((e,t,r,a)=>{var n=document.createElement("link");n.rel="stylesheet",n.type="text/css",n.onerror=n.onload=o=>{if(n.onerror=n.onload=null,"load"===o.type)r();else{var i=o&&("load"===o.type?"missing":o.type),c=o&&o.target&&o.target.href||t,d=new Error("Loading CSS chunk "+e+" failed.\n("+c+")");d.code="CSS_CHUNK_LOAD_FAILED",d.type=i,d.request=c,n.parentNode.removeChild(n),a(d)}},n.href=t,document.head.appendChild(n)})(e,n,t,r)})),n={121:0},c.f.miniCss=(e,t)=>{n[e]?t.push(n[e]):0!==n[e]&&{131:1,168:1,186:1,198:1,249:1,294:1,327:1,350:1,380:1,412:1,442:1,460:1,486:1,576:1,580:1,616:1,621:1,773:1,819:1,824:1,877:1,915:1}[e]&&t.push(n[e]=a(e).then((()=>{n[e]=0}),(t=>{throw delete n[e],t})))},(()=>{var e={121:0};c.f.j=(t,r)=>{var a=c.o(e,t)?e[t]:void 0;if(0!==a)if(a)r.push(a[2]);else if(/^([16]21|198|294|350|380|773)$/.test(t))e[t]=0;else{var n=new Promise(((r,n)=>a=e[t]=[r,n]));r.push(a[2]=n);var o=c.p+c.u(t),i=new Error;c.l(o,(r=>{if(c.o(e,t)&&(0!==(a=e[t])&&(e[t]=void 0),a)){var n=r&&("load"===r.type?"missing":r.type),o=r&&r.target&&r.target.src;i.message="Loading chunk "+t+" failed.\n("+n+": "+o+")",i.name="ChunkLoadError",i.type=n,i.request=o,a[1](i)}}),"chunk-"+t,t)}},c.O.j=t=>0===e[t];var t=(t,r)=>{var a,n,[o,i,d]=r,l=0;if(o.some((t=>0!==e[t]))){for(a in i)c.o(i,a)&&(c.m[a]=i[a]);if(d)var s=d(c)}for(t&&t(r);l<o.length;l++)n=o[l],c.o(e,n)&&e[n]&&e[n][0](),e[n]=0;return c.O(s)},r=self.webpackChunkWebComponents=self.webpackChunkWebComponents||[];r.forEach(t.bind(null,0)),r.push=t.bind(null,r.push.bind(r))})()})();
//# sourceMappingURL=prefix-runtime.a0c870bab402ccbf.bundle.js.map