.selectwidthauto{display:inline-block!important;margin:5px;width:auto!important}input.input-validation-error{border:1px solid #e80c4d}.dark-mode .fillintheblank_section input[type=text]{background-color:#333;color:unset}
/*# sourceMappingURL=prefix-294.38995dcf76812d5a.css.map*/