.poll .poll-question{padding-bottom:10px}.poll-options>label{margin-right:5px}.poll .progress{height:25px;margin-top:-20px;width:350px}.poll .progress .progress-bar{color:#333;height:30px}.poll-text{display:inline-block;line-height:22px;vertical-align:text-bottom}.results-container{display:grid;grid-template-columns:2em 90%}.poll-option{display:inline}.poll-option .para{display:inline-block;padding-left:.5rem}.poll-results{margin-top:.5rem}
/*# sourceMappingURL=prefix-168.07f19b71041a3105.css.map*/