.evalCont{background-color:#f5f5f5;border:1px solid #ccc;border-radius:4px;color:#333;font-family:monospace;line-height:1.5em;padding:9.5px}.anno,.evalCont{display:block;width:auto}.anno{background-color:#e2ebf6;border:1px solid #a6b3b6;box-shadow:-2px 1px 5px #ccc;font-family:sans-serif;left:80%;max-height:100px;max-width:250px;min-width:150px;overflow:scroll;position:absolute;top:5px}.currentStepDiv{position:relative}
/*# sourceMappingURL=prefix-824.271ae6cff6401ed5.css.map*/