.datafiletextfield{border:1px solid var(--componentBorderColor);display:block}.datafile_caption{background:var(--componentBgColor);border:1px solid var(--componentBorderColor);border-radius:4px 4px 0 0;color:var(--bodyFont);display:block;font-style:italic;margin-bottom:-1px;padding:0 .2em;width:fit-content}.datafile_caption code{background:none;border:none}.datafile pre{border:1px solid var(--componentBorderColor);font-size:90%;line-height:1.5em;max-height:30em;max-width:100%;overflow:auto;white-space:pre}.datafile_hidden{display:none}.datafile img{border:1px solid var(--componentBorderColor);display:block;margin:0}.datafile_caption code.code-inline{background:inherit;border:0;color:inherit}
/*# sourceMappingURL=prefix-580.22958d4327b0425a.css.map*/