.ptx-runestone-container .rsdraggable,.runestone-sphinx .rsdraggable{background-color:var(--questionBgColor,#efefff);border:1px solid var(--componentBorderColor,#efefff);box-sizing:border-box;display:inline;float:left;font-family:monospace;font-size:120%;margin-left:1%;margin-right:1%;margin-top:5px;min-width:28%;padding-bottom:10px;text-align:left;vertical-align:top}.ptx-runestone-container .dnd-button-container,.runestone-sphinx .dnd-button-container{display:flex;justify-content:center;margin-top:1em;width:100%}.ptx-runestone-container .drag-button,.runestone-sphinx .drag-button{display:inline-block}.ptx-runestone-container .drag-reset,.runestone-sphinx .drag-reset{background-color:#474949;color:#fff}.ptx-runestone-container .drag-reset:hover,.runestone-sphinx .drag-reset:hover{background-color:#000;color:#fff}.ptx-runestone-container .drop-incorrect,.runestone-sphinx .drop-incorrect{background-color:var(--draggableIncorrectBg,#f2dede)!important;border:1px solid var(--draggableIncorrectBorder,red)!important}.ptx-runestone-container .drag-reset:enabled,.runestone-sphinx .drag-reset:enabled{background-color:#474949;color:#fff}.ptx-runestone-container .draggable-drag,.runestone-sphinx .draggable-drag{background-color:var(--draggableBgColor);border:1px solid #d3d3d3;border-radius:10px;cursor:grab;display:block;margin-top:5px;padding:10px}.ptx-runestone-container .draggable-feedback,.runestone-sphinx .draggable-feedback{clear:both;text-align:left!important}.ptx-runestone-container .draggable-drop,.runestone-sphinx .draggable-drop{background-color:var(--dropableBgColor);border:1px solid #d3d3d3;border-radius:10px;cursor:grab;display:block;margin-top:5px;padding:10px}.ptx-runestone-container .draggable-container,.runestone-sphinx .draggable-container{text-align:center}.ptx-runestone-container .possibleDrop,.runestone-sphinx .possibleDrop{background-color:#ccc;border-color:#000}.ptx-runestone-container .draggable-drop .MathJax,.ptx-runestone-container .draggable-drop .process-math,.runestone-sphinx .draggable-drop .MathJax,.runestone-sphinx .draggable-drop .process-math{cursor:not-allowed;pointer-events:none}:root{--draggableBgColor:#efefef;--dropableBgColor:#ddd;--draggableIncorrectBorder:red;--draggableIncorrectBg:#f2dede}:root.dark-mode,[data-theme=dark]{--draggableBgColor:#393434;--dropableBgColor:#2a2d30;--draggableIncorrectBorder:red;--draggableIncorrectBg:#6d2d2d}
/*# sourceMappingURL=prefix-621.1beebe7935b0310d.css.map*/