/********************* Colour reference chart****************
elememt ********* property ********* colour ********

webgl_container  background-color    #fcf8e3

*/

/* Major sections */
.webgl_container {
  background-color: #fcf8e3;
  box-sizing: border-box;
  width: 100%;
  display: block;
  margin-bottom: 1em;
}

.webgl_cmds {
  box-sizing: border-box;
  width: 100%;
  text-align: center;
  padding: 6px;
}

.webgl_row2 {
  box-sizing: border-box;
  width: 100%;
  padding: 0px 4px 0px 4px;
  display: block;
  height: 550px;
}

.webgl_editors {
  box-sizing: border-box;
  float: left;
  display: block;
  visibility: visible;
  width: 50%;
}

.webgl_canvas {
  box-sizing: border-box;
  float: left;
  display: block;
  visibility: visible;
  font-size: 12px;
  font-weight: normal;
  padding-left: 5px;
  width: 50%;
}

.webgl_output {
  box-sizing: border-box;
  width: 100%;
  padding: 4px;
  margin-top: 6px;
  float: left;
}

/* Details within each section */
/* Tabbed titles, ul horizontal list */
.webgl_nav_tabs {
  height: 31px;
  width: 100%;
  margin: 0px;
  padding: 6px 0px 0px 0px;
  background-color: #CFCCB8;
  border-color: #fbeed5;
  border-top-left-radius: 4px;
  border-top-right-radius: 4px;
  list-style-type: none;
  overflow: hidden;
  display: inline-flex;
}

.webgl_nav_tabs li {
  border-top-left-radius: 4px;
  border-top-right-radius: 4px;
  margin: 0px 0px 0px 4px;
  padding: 4px 1px 4px 4px;
  list-style-type: none;
  position: relative;
  float: left;
  background-color: #E6E2CC;
  font-size: 9pt;
}

.webgl_nav_tabs li > a {
  padding: 8px 8px 8px 8px;
  color: #555;
  text-decoration: none;
}

.webgl_nav_tabs > li > a:hover {
  border-color: #b4b19d;
  border-bottom-width: 0;
}

.webgl_nav_tabs li.active {
  background-color: #ffffff;
  list-style-type: none;
  list-style-image: none;
}

/* Webgl editors */
.webgl_tab_content {
  width: 100%;
  position: absolute;
  clear: both;
  margin: 0px 0px 20px 0px;
  border: 0px;
  color: #c09853;
  background-color: #fcf8e3;
  z-index: 1;
}

.webgl_tab_content.active {
  z-index: 2;
}

.webgl_tab_content > * {
  margin-left: auto;
  margin-right: auto;
  /*	position:relative; */
}

.webgl_tab_content .clearfix {
  position: initial;
}

.webgl_tab_editor {
  width: 100%;
  position: absolute;
  clear: both;
  padding: 10px 4px 10px 4px;
  margin: 0px 0px 20px 0px;
  border: 0px;
  color: #c09853;
  background-color: #fcf8e3;
}

/* span that contains the tabbed text editors */
.webgl_code {
  width: 100%;
  float: left;
}

.webgl_tab_contents_text {
  width: 100%;
  position: absolute;
  clear: both;
  padding: 10px 4px 10px 4px;
  margin: 0px 0px 20px 0px;
  border: 0px;
  color: #c09853;
  background-color: #fcf8e3;
  z-index: 1;
}

.webgl_errorMessages {
  color: red;
}

.webgl_warningMessages {
  color: rgb(170, 34, 164);
}

.webgl_infoMessages {
  color: rgb(24, 144, 64);
}

.webgl_output_div {
  width: 100%;
  margin: 2px;
  border: 1px solid lightgrey;
  padding-bottom: 4px;
  padding-top: 4px;
  padding-left: 4px;
  background-color: white;
  overflow: auto;
  height: 14em;
  font-size: 12px;
  font-weight: normal;
}

.webgl_editor_highlight {
  background-color: #EEEEEE; /* very light grey */
}

/* 
Removed as these rules apply to entire browser window and not just webgl interactives
::-webkit-scrollbar {
    -webkit-appearance: none;
    width: 7px;
}
::-webkit-scrollbar-thumb {
    border-radius: 4px;
    background-color: rgba(0,0,0,.5);
    -webkit-box-shadow: 0 0 1px rgba(255,255,255,.5);
}
*/

.webgl_btn {
  display: inline-block;
  padding: 3px 6px;
  margin-bottom: 0;
  background-color: thistle;
  font-size: 12px;
  font-weight: normal;
  text-align: center;
  white-space: nowrap;
  vertical-align: middle;
  cursor: pointer;
  border: 1px solid transparent;
  border-radius: 4px;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
}

.webgl_checkbox {
}

.webgl_tabbed_editor {
  width: 100%;
  resize: none;
  font-family: monospace, sans-serif;
  font-size: 9pt;
  font-style: normal;
}

.canvas3D {
  width: 100%;
  height: 100%;
}

.CodeMirror.webgl-default {
  border: 1px solid #eee;
  font-family: monospace, sans-serif;
  font-size: 8pt;
  font-style: normal;
}

button.btn.webgl-default {
  font-size: 8pt;
}


